/*****************************************************************************
	AmyEdit - A GTKMM LaTeX Editor
	Copyright (C) 2004-2005 Rob Page 
	email:  page <dot> rob <at> gmail <dot> com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "AmyEdit.hh"
#include "AboutDialog.hh"
#include "FileDialog.hh"
#include "Preferences.hh"
#include "Utils.hh"

#include <gtkmm/toolbar.h>
#include <gtkmm/box.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/stock.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/frame.h>
#include <gtkmm/separator.h>

#include <unistd.h>

/* My Signals */
// signals when there is a file available
SigC::Signal1<void,bool> signal_file_open;
// signals an update to preferences
SigC::Signal0<void> signal_preferences_update;
/**************/

AmyEdit* AmyEdit::smInstance=NULL;

AmyEdit* AmyEdit::instance()
{
	static bool running = false;
	if (smInstance == NULL && running == false)
	{
		running = true;
		smInstance = new AmyEdit();
		running = false;
	}
	return smInstance;
}

void AmyEdit::goodnight()
{
	if (smInstance != NULL)
	{
		delete smInstance;
	}
}

AmyEdit::AmyEdit() : 
	Gtk::Window(Gtk::WINDOW_TOPLEVEL)
{
	
	edittabs = Gtk::manage(new EditTabs());

	/*********************
	 * Menu Construction *
	 *********************/

	// perm_ag = action group for always active items
	// need_file_ag = action group for only active when a file is open */

	uiman = Gtk::UIManager::create();
	Glib::RefPtr<Gtk::ActionGroup> perm_ag = Gtk::ActionGroup::create();
	need_file_ag = Gtk::ActionGroup::create();
	perm_ag->add(Gtk::Action::create("MenuFile", _("_File")) );
	perm_ag->add(Gtk::Action::create("New", Gtk::Stock::NEW), sigc::mem_fun(*this, &AmyEdit::on_new));
	perm_ag->add(Gtk::Action::create("Open", Gtk::Stock::OPEN), sigc::mem_fun(*this, &AmyEdit::on_open));
	need_file_ag->add(Gtk::Action::create("Save", Gtk::Stock::SAVE), sigc::mem_fun(*this, &AmyEdit::on_save)); 
	need_file_ag->add(Gtk::Action::create("Save As", Gtk::Stock::SAVE_AS), sigc::mem_fun(*this, &AmyEdit::on_save_as));
	need_file_ag->add(Gtk::Action::create("Save All", Gtk::Stock::DND_MULTIPLE, _("Save All")), sigc::mem_fun(*this, &AmyEdit::on_save_all));
	need_file_ag->add(Gtk::Action::create("Close", Gtk::Stock::CLOSE), sigc::mem_fun(*this, &AmyEdit::on_close));
	need_file_ag->add(Gtk::Action::create("Save as Template", Gtk::Stock::SAVE_AS, _("Save as Template")), sigc::mem_fun(*this, &AmyEdit::on_save_as_template));
	need_file_ag->add(Gtk::Action::create("Export", Gtk::Stock::SAVE_AS, _("Export")), sigc::mem_fun(*this, &AmyEdit::on_export));
	perm_ag->add(Gtk::Action::create("Quit", Gtk::Stock::QUIT), sigc::mem_fun(*this, &AmyEdit::on_quit));

	perm_ag->add(Gtk::Action::create("MenuEdit", _("_Edit")));
	need_file_ag->add(Gtk::Action::create("Undo", Gtk::Stock::UNDO), Gtk::AccelKey("<control>Z"), sigc::mem_fun(*edittabs, &EditTabs::Undo));
	need_file_ag->add(Gtk::Action::create("Redo", Gtk::Stock::REDO), Gtk::AccelKey("<control><shift>Z"), sigc::mem_fun(*edittabs, &EditTabs::Redo));
	need_file_ag->add(Gtk::Action::create("Cut", Gtk::Stock::CUT), sigc::mem_fun(*edittabs, &EditTabs::CutText));
	need_file_ag->add(Gtk::Action::create("Copy", Gtk::Stock::COPY), sigc::mem_fun(*edittabs, &EditTabs::CopyText));
	need_file_ag->add(Gtk::Action::create("Paste", Gtk::Stock::PASTE), sigc::mem_fun(*edittabs, &EditTabs::PasteText));
	need_file_ag->add(Gtk::Action::create("Delete", Gtk::Stock::DELETE), sigc::mem_fun(*edittabs, &EditTabs::DeleteText));
	need_file_ag->add(Gtk::Action::create("Find", Gtk::Stock::FIND), sigc::mem_fun(*this, &AmyEdit::on_find));

	perm_ag->add(Gtk::Action::create("MenuView", _("_View")));
	need_file_ag->add(Gtk::Action::create("Preview", Gtk::Stock::PRINT_PREVIEW, _("Preview")), Gtk::AccelKey("<control>P"), sigc::mem_fun(*this, &AmyEdit::on_preview));

	perm_ag->add(Gtk::Action::create("MenuTools", _("_Tools")));
	need_file_ag->add(Gtk::Action::create("SpellCheck", Gtk::Stock::SPELL_CHECK), sigc::mem_fun(*this, &AmyEdit::on_spellcheck));
	need_file_ag->add(Gtk::Action::create("WordCount", Gtk::Stock::PROPERTIES,  _("WordCount")), sigc::mem_fun(*this, &AmyEdit::on_word_count));
	perm_ag->add(Gtk::Action::create("Preferences", Gtk::Stock::PREFERENCES), sigc::mem_fun(*this, &AmyEdit::on_preferences));

	perm_ag->add(Gtk::Action::create("MenuHelp", _("_Help")));
	perm_ag->add(Gtk::Action::create("About", Gtk::Stock::DIALOG_INFO, _("About")), sigc::mem_fun(*this, &AmyEdit::on_about));

	need_file_ag->set_sensitive(false); 
	uiman->insert_action_group(perm_ag);
	uiman->insert_action_group(need_file_ag);
	add_accel_group(uiman->get_accel_group());

	signal_file_open.connect(sigc::mem_fun(*this, &AmyEdit::on_file_open));

	Glib::ustring ui_info =
		"<ui>"
		"  <menubar name='MenuBar'>"
		"    <menu action='MenuFile'>"
		"      <menuitem action='New'/>"
		"      <menuitem action='Open'/>"
		"      <menuitem action='Save'/>"
		"      <menuitem action='Save As'/>"
		"      <menuitem action='Save All'/>"
		"      <menuitem action='Close'/>"
		"      <separator/>"
		"      <menuitem action='Save as Template'/>"
		"      <menuitem action='Export'/>"
		"      <separator/>"
		"      <placeholder name='RecentFiles'/>"
		"      <separator/>"
		"      <menuitem action='Quit'/>"
		"    </menu>"
		"    <menu action='MenuEdit'>"
		"      <menuitem action='Undo'/>"
		"      <menuitem action='Redo'/>"
		"      <separator/>"
		"      <menuitem action='Cut'/>"
		"      <menuitem action='Copy'/>"
		"      <menuitem action='Paste'/>"
		"      <menuitem action='Delete'/>"
		"      <separator/>"
		"      <menuitem action='Find'/>"
		"    </menu>"
		"    <menu action='MenuView'>"
		"      <menuitem action='Preview'/>"
		"    </menu>"
		"    <menu action='MenuTools'>"
		"      <menuitem action='SpellCheck'/>"
		"      <menuitem action='WordCount'/>"
		"      <separator/>"
		"      <menuitem action='Preferences'/>"
		"    </menu>"
		"    <menu action='MenuHelp'>"
		"      <menuitem action='About'/>"
		"    </menu>"
		"  </menubar>"
		"  <toolbar  name='ToolBar'>"
		"    <toolitem action='New'/>"
		"    <toolitem action='Open'/>"
		"    <toolitem action='Save'/>"
		"    <separator/>"
		"    <toolitem action='Preview'/>"
		"    <toolitem action='Export'/>"
		"  </toolbar>"
		"</ui>";

	uiman->add_ui_from_string(ui_info);

	recent_files_update(false);

	/*****************
	 * Add tooltips! *
	 *****************/

	Gtk::Tooltips *tooltips = Gtk::manage(new Gtk::Tooltips());
	Gtk::Toolbar* toolbar = static_cast<Gtk::Toolbar*>(uiman->get_widget("/ui/ToolBar"));
	for (int i=0; i < toolbar->get_n_items(); i++)
	{
		Gtk::ToolItem* toolitem = static_cast<Gtk::ToolItem*>(toolbar->get_nth_item(i));
		toolitem->set_tooltip(*tooltips, toolitem->get_name());

		// Set priority so they get text when in horiz mode
		toolitem->set_is_important(true);
	}
	tooltips->enable();
	toolbar->set_tooltips(true);

	/*******************
	 * Main Area Setup *
	 *******************/

	Gtk::Frame *edittabsframe = Gtk::manage(new Gtk::Frame());
	edittabsframe->add(*edittabs);

	Gtk::Frame *outputswframe = Gtk::manage(new Gtk::Frame());
	Gtk::ScrolledWindow *outputsw = Gtk::manage(new Gtk::ScrolledWindow());
	output.modify_font(Pango::FontDescription("monospace 8"));
	output.set_justification(Gtk::JUSTIFY_LEFT);
	output.set_editable(false);
	outputsw->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);
	outputsw->add(output);
	outputswframe->add(*outputsw);

	paneview.pack1(*edittabsframe);
	paneview.pack2(*outputswframe,Gtk::SHRINK);
	paneview.set_position(65535);

	/***************************
	 * Add Everything and Show *
	 ***************************/


	Gtk::VBox *vbox = Gtk::manage(new Gtk::VBox(false, 0));
	vbox->pack_start(*uiman->get_widget("/MenuBar"), Gtk::PACK_SHRINK);
	vbox->pack_start(*uiman->get_widget("/ToolBar"), Gtk::PACK_SHRINK);
	vbox->pack_start(paneview, Gtk::PACK_EXPAND_WIDGET);
	vbox->pack_start(*Gtk::manage(new Gtk::HSeparator()), Gtk::PACK_SHRINK);
	vbox->pack_start(statusbar, Gtk::PACK_SHRINK);
	set_title("AmyEdit " VERSION);
	PrefStruct* prefs = PrefStruct::instance();
	set_default_size(prefs->get<int>("General", "window_width"), prefs->get<int>("General", "window_height"));
	add(*vbox);

	show_all();
}

AmyEdit::~AmyEdit()
{ 
	PrefStruct* prefs = PrefStruct::instance();
	int width, height;
	get_size(width,height);
	prefs->set("General", "window_width" , width );
	prefs->set("General", "window_height", height);
	PrefStruct::goodnight();
}

void AmyEdit::open(Glib::ustring filename)
{
	edittabs->OpenTab(filename,true);
}

void AmyEdit::open_recent(int filenumber)
{
	std::vector<Glib::ustring> recent_files = PrefStruct::instance()->get<Glib::ArrayHandle<Glib::ustring> >("General", "recent_files");
	edittabs->OpenTab(recent_files.at(filenumber), false);
}

bool AmyEdit::on_delete_event(GdkEventAny* event)
{

	if (edittabs->CloseAllTabs())
		return false;
	return true;

}

void AmyEdit::on_new()
{  
	statusbar.setReady(false);
	edittabs->NewTab();
	statusbar.setReady(true);
}

void AmyEdit::on_open()
{
	statusbar.setReady(false);
	edittabs->OpenTab();
	statusbar.setReady(true);
}

void AmyEdit::on_save()
{  
	statusbar.setReady(false);
	edittabs->SaveCurrentTab();
	statusbar.setReady(true);
}

void AmyEdit::on_save_as()
{  
	statusbar.setReady(false);
	edittabs->SaveCurrentTabAs();
	statusbar.setReady(true);
}

void AmyEdit::on_save_all()
{
	statusbar.setReady(false);
	edittabs->SaveAllTabs();
	statusbar.setReady(true);
}

void AmyEdit::on_close()
{
	statusbar.setReady(false);
	edittabs->CloseTab();
	statusbar.setReady(true);
}  

void AmyEdit::on_save_as_template()
{  
	statusbar.setReady(false);
	edittabs->SaveCurrentTabAsTemplate();
	statusbar.setReady(true);
}


void AmyEdit::on_export()
{
	statusbar.setReady(false);
	Glib::ustring filename;

	if((filename = edittabs->Compile())!="")
	{
		FileDialog fdialog(Glib::ustring(_("Export to...")),Gtk::FILE_CHOOSER_ACTION_SAVE);
		int result=fdialog.run();

		if(result==Gtk::RESPONSE_OK)
		{
			bool writefile = true;
			int mode = access(fdialog.get_filename().c_str(),F_OK);
			if (!mode)
			{
				int result=Amy::Message(_("File ")+fdialog.get_filename()+_(" already exists"), _("Are you sure you want to overwrite it?"), Amy::OK_CANCEL);
				if(result!=Gtk::RESPONSE_OK)
					writefile=false;
			}
			if (writefile)
			{
				std::string sout,serr;
				int exitstat;
				Glib::spawn_command_line_sync(Glib::ustring("cp ")+ filename + " " + fdialog.get_filename(), &sout, &serr, &exitstat);
				if (!exitstat)
				{
					Amy::Message(_("Export Successful"), _("The file ") + fdialog.get_filename()+_(" has been successfully exported"));
				}
				else
				{
					Amy::Message(_("Export Failed!") , sout+serr, Amy::ERROR);
				}
			}

		}
	}
	statusbar.setReady(true);
}


void AmyEdit::on_quit()
{  
	statusbar.setReady(false);
	if (edittabs->CloseAllTabs())
		this->hide();
	statusbar.setReady(true);
}

void AmyEdit::on_find()
{
	statusbar.setReady(false);
	edittabs->FindNext();
	statusbar.setReady(true);
}

void AmyEdit::on_preview()
{
	statusbar.setReady(false);

	Glib::ustring filename = edittabs->Compile();

	if(filename!="")
	{
		Glib::spawn_command_line_async("sh -c '" + Amy::get_command(Amy::PreviewCommand,filename) + "'");
	}

	statusbar.setReady(true);
}

void AmyEdit::on_spellcheck()
{  
	statusbar.setReady(false);
	edittabs->SpellCheck();
	statusbar.setReady(true);
}

void AmyEdit::on_word_count()
{  
	statusbar.setReady(false);

	if(edittabs->SaveTmpTex())
	{
		int exitstat;
		std::string sout;
		std::string serr;
		Glib::ustring filename=Glib::build_filename(PrefStruct::instance()->get<Glib::ustring>("General", "config_dir") , "temp.tex");
		Glib::spawn_command_line_sync("sh -c 'wc -w " + filename + " | sed -e s!" + filename + "!!'",&sout,&serr,&exitstat);
		Amy::Message(_("Word Count:"), sout, Amy::INFO);
	}
	statusbar.setReady(true);

}


void AmyEdit::on_preferences()
{

	statusbar.setReady(false);

	Preferences dialog;
	int result=dialog.run();
	if(result==Gtk::RESPONSE_OK)
	{
		signal_preferences_update();
	}
	statusbar.setReady(true);

}

void AmyEdit::on_about()
{ 
	statusbar.setReady(false);

	AboutDialog dialog;
	dialog.run();
	statusbar.setReady(true);

}

void AmyEdit::on_file_open(bool file)
{
	need_file_ag->set_sensitive(file);
}

void AmyEdit::set_output_text(Glib::ustring text)
{
	output.get_buffer()->set_text(text);
}

void AmyEdit::pop_up_output()
{
	paneview.set_position(paneview.property_max_position()-200);
}

void AmyEdit::recent_files_update(bool unmerge_old_id)
{
	if (unmerge_old_id)
		uiman->remove_ui(recent_files_id);

	Glib::ustring test=
		"<ui>"
		" <menubar name='MenuBar'>"
		"    <menu action='MenuFile'>"
		"      <placeholder name='RecentFiles'>";

	recent_files_ag = Gtk::ActionGroup::create();

	PrefStruct* prefs = PrefStruct::instance();
	std::vector<Glib::ustring>recent_files = prefs->get<Glib::ArrayHandle<Glib::ustring> >("General", "recent_files");
	
	unsigned int i;
	for(i=0;i<recent_files.size();i++)
	{
		Glib::ustring current_recent_file = recent_files.at(i);
		if (current_recent_file!="")
		{
			recent_files_ag->add(Gtk::Action::create(Glib::ustring("File"+ Amy::itoa(i)), 
			                     Glib::ustring("_"+ Amy::itoa(i+1) + " " + Glib::path_get_basename(current_recent_file))),
			                     sigc::bind<int>(sigc::mem_fun(*this, &AmyEdit::open_recent),i));
			test+="<menuitem action='File" + Amy::itoa(i) + "'/>";
		}
	}

	test+= 
		"      </placeholder>"
		"    </menu>"
		"  </menubar>"
		"</ui>";

	uiman->insert_action_group(recent_files_ag);

	recent_files_id = uiman->add_ui_from_string(test);
}

void AmyEdit::add_recent_file (Glib::ustring filename)
{
	const unsigned int recent_file_max_size = 5;
	PrefStruct* prefs = PrefStruct::instance();
	
	std::list<Glib::ustring> recent_files = prefs->get<Glib::ArrayHandle<Glib::ustring> >("General", "recent_files");
	std::list<Glib::ustring>::iterator recent_files_iterator;

	for(recent_files_iterator = recent_files.begin(); recent_files_iterator != recent_files.end(); recent_files_iterator++ ) 
	{
		if (*recent_files_iterator==filename)
		{
			recent_files.erase(recent_files_iterator);
			break;
		}
	}
	
	recent_files.push_front(filename);

	if (recent_files.size() > recent_file_max_size)
		recent_files.pop_back();

	prefs->set("General", "recent_files", recent_files);
	recent_files_update();
}

