/*****************************************************************************
  AmyEdit - A GTKMM LaTeX Editor
  Copyright (C) 2004-2005 Rob Page
email:  page <dot> rob <at> gmail <dot> com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "EditTabs.hh"
#include "NewDialog.hh"
#include "FileDialog.hh"
#include "FindDialog.hh"
#include "AmyEdit.hh"
#include "SpellDialog.hh"
#include "Utils.hh"

#include <gtksourceview/gtksourcebuffer.h>
#include <gtksourceviewmm/sourcelanguage.h>
#include <gtksourceviewmm/sourcelanguagesmanager.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#include <gtksourceview/gtksourceiter.h>

#include <gtkmm/stock.h>

#include <aspell.h>
#include <string>

EditTabs::EditTabs() : Gtk::Notebook()
{

	SpellDialog::init();

	set_show_tabs(true);
	set_show_border(true);
	set_tab_pos(Gtk::POS_TOP);
	set_scrollable(true);

	extern SigC::Signal0<void> signal_preferences_update;

	signal_switch_page().connect(sigc::mem_fun(*this, &EditTabs::on_switching_page));
	signal_preferences_update.connect(sigc::mem_fun(*this, &EditTabs::on_preferences_update));
}

EditTabs::~EditTabs()
{
	SpellDialog::deinit();
}

void EditTabs::NewTab()
{
	NewDialog dialog;

	int result=dialog.run();
	if (result==Gtk::RESPONSE_OK)
	{

		Page *newpage= Gtk::manage(new Page(this));

		newpage->filename=Glib::ustring("");
		newpage->get_buffer()->set_text(dialog.getDoc());

		// This needed to guarantee set_modified is changed, 
		// and thus signal handler gets called.
		newpage->get_buffer()->set_modified(true);  
		newpage->get_buffer()->set_modified(false);
		newpage->get_child()->grab_focus();
		newpage->get_buffer()->place_cursor(newpage->get_buffer()->begin());
		set_current_page(page_num(*newpage));
	}
	else if (result==Gtk::RESPONSE_NONE)
	{
		Page *newpage= Gtk::manage(new Page(this));
		newpage->filename=Glib::ustring("");
		newpage->get_buffer()->set_modified(true);
		newpage->get_buffer()->set_modified(false);
		newpage->get_child()->grab_focus();
		newpage->get_buffer()->place_cursor(newpage->get_buffer()->begin());
		set_current_page(page_num(*newpage));
	}

}

void EditTabs::OpenTab()
{
	FileDialog dialog(_("Open File..."),Gtk::FILE_CHOOSER_ACTION_OPEN);

	int result=dialog.run();
	if (result==Gtk::RESPONSE_OK)
	{
		OpenTab(dialog.get_filename());
	}
}

void EditTabs::OpenTab(Glib::ustring filename, bool open_if_not_found)
{
	if (!Glib::path_is_absolute(filename))
	{
		filename = Glib::build_filename(Glib::get_current_dir(), filename);
	}

	if (PrefStruct::instance()->get<bool>("General", "warn_about_filenames_with_spaces") && Glib::path_get_basename(filename).find(' ')!=Glib::ustring::npos)
	{
		int result = Amy::Message(_("Filename contains spaces!"), 
				_("LaTeX does not support spaces in filenames,\n"
				"You are attempting to open the file: ") + filename + _("\n"
				"Do you still want to open it?\n"
				"You can turn this warning off in preferences"),
				Amy::OK_CANCEL);
		if (result == Gtk::RESPONSE_CANCEL)
			return;
	}

	bool found = false;
	if (Glib::file_test(filename,Glib::FILE_TEST_EXISTS))
	{
		found = true;
	}

	if (found || open_if_not_found)
	{
		try 
		{
			Glib::RefPtr<Gtk::SourceBuffer> buffer = is_buffer_open(filename);
			Page *newpage;	

			if (buffer){
				newpage = Gtk::manage(new Page(this, buffer));
				newpage->filename=filename;
				newpage->encoding=PrefStruct::instance()->get<Glib::ustring>("General", "file_encoding");
				bool modified = newpage->get_buffer()->get_modified();
				newpage->get_buffer()->set_modified(true);
				newpage->get_buffer()->set_modified(false);
				newpage->get_buffer()->set_modified(modified);

			} else {
				Glib::ustring temp = "";

				if (found){
					Glib::RefPtr<Glib::IOChannel> file = Glib::IOChannel::create_from_file(filename, "r");
					file->set_encoding(PrefStruct::instance()->get<Glib::ustring>("General", "file_encoding"));
					file->read_to_end(temp);
				}

				newpage= Gtk::manage(new Page(this));

				newpage->filename=filename;
				newpage->encoding=PrefStruct::instance()->get<Glib::ustring>("General", "file_encoding");
				newpage->get_buffer()->insert_at_cursor(temp);

				// Ensure modified toggles, so that tab labels get updated, etc.
				newpage->get_buffer()->set_modified(true);
				newpage->get_buffer()->set_modified(false);
			}

			newpage->get_child()->grab_focus();
			newpage->get_buffer()->place_cursor(newpage->get_buffer()->get_iter_at_line(0));

			set_current_page(page_num(*newpage));
			AmyEdit::instance()->add_recent_file(filename);

		} catch(Glib::IOChannelError error) {
			Amy::Message(_("Cannot Open File"),  _("An error occurred whilst attempting to open the file ") + filename + _(".\nThe system reported this error as: ") + error.what(), Amy::ERROR);
		} catch(Glib::ConvertError error) {
			Amy::Message(_("Conversion Error"), _("An error occurred whilst converting the file ") + filename + _(" as ") + PrefStruct::instance()->get<Glib::ustring>("General", "file_encoding") + _(". If you are using UTF8 then this file is probably not a LaTeX source file.\nThe system reported this error as: ") + error.what(), Amy::ERROR);
		} catch(Glib::FileError error) {
			Amy::Message(_("File Error"), _("An error occurred whilst attempting to open the file ") + filename + _(".\nThe system reported this error as: ") + error.what(), Amy::ERROR);
		}
	}
	else
	{
		Amy::Message(_("Cannot Open File"), _("The file ") + filename + _(" was not found on your system"), Amy::ERROR);
	}
}

bool EditTabs::SaveCurrentTab()
{
	Page *curtab = get_current_tab();
	if (!curtab)
	{
		Amy::Message(_("Save Failed"), _("It appears you do not have a document open at the moment"), Amy::ERROR);
		return false;
	}
	else
	{

		if (curtab->filename==Glib::ustring(""))
		{
			return SaveCurrentTabAs();
		}
		else
		{
			return WriteTab();
		}
	}
}

bool EditTabs::SaveCurrentTabAs()
{
	// Check there's a Tab
	Page *curtab = get_current_tab();

	if (curtab==NULL)
	{
		Amy::Message(_("Save Failed"), _("It appears you do not have a document open at the moment"), Amy::ERROR);
		return false;
	}
	else
	{
		// Open Save As Dialog
		FileDialog dialog(Glib::ustring(_("Save File As...")),Gtk::FILE_CHOOSER_ACTION_SAVE);
		int result;

		result=dialog.run();

		if(PrefStruct::instance()->get<bool>("General", "warn_about_filenames_with_spaces"))
		{
			while (result==Gtk::RESPONSE_OK){

				if (Glib::path_get_basename(dialog.get_filename()).find(' ')!=Glib::ustring::npos)
				{
					int aresult = Amy::Message(_("Filename contains spaces!"), 
							_("LaTeX does not support spaces in filenames,\n"
							"You are attempting to save the file: ") + 
							Glib::path_get_basename(dialog.get_filename()) + _("\n"
							"Do you still want to save it?\n"
							"You can turn this warning off in preferences"),
							Amy::OK_CANCEL);
					if (aresult == Gtk::RESPONSE_OK)
						break;

					result=dialog.run();
				} else {
					break;
				}
			}
		}

		if(result==Gtk::RESPONSE_OK)
		{
			bool writefile = true;
			int mode = access(dialog.get_filename().c_str(),F_OK);
			if (!mode)
			{
				int result = Amy::Message(_("Overwrite ") +dialog.get_filename()+ 
						_(" existing file?"), 
						_("A file with the name you gave already exists\n"
						"Do you wish to overwrite it?"),
						Amy::OK_CANCEL);
				if(result!=Gtk::RESPONSE_OK)
					writefile=false;

			}

			// Update internal data
			if (writefile)
			{
				curtab->filename=dialog.get_filename();
				// Set Tab and WriteFile
				curtab->tablabel->set_text(TabNameFromFile(curtab->filename));
				return WriteTab();
			}
		}
		return false;
	}
}

bool EditTabs::WriteTab()
{
	Page *curtab = get_current_tab();
	if (curtab==NULL)
	{
		Amy::Message(_("Save Failed"), _("It appears you do not have a document open at the moment, so this dialog should never be seen"), Amy::ERROR);
		return false;
	}
	else
	{
		try
		{
			Glib::RefPtr<Glib::IOChannel>outfile = Glib::IOChannel::create_from_file(curtab->filename,"w");
			outfile->write(curtab->get_buffer()->get_text());
			outfile->flush();
			curtab->get_buffer()->set_modified(false);
			AmyEdit::instance()->add_recent_file (curtab->filename);

		}
		catch(Glib::FileError error)
		{
			Amy::Message(_("Save Failed"), _("An error occurred whilst attempting to save the file ") + curtab->filename + _(".\nThe system reported this error as: ") + error.what(), Amy::ERROR);
			return false;
		}
	}


	return true;
}

Glib::ustring EditTabs::Compile()
{
	Glib::ustring filename = "";

	Page *curtab = get_current_tab();
	if (curtab==NULL)
	{
		Amy::Message(_("Markup Failed"), _("It appears you have nothing to markup at present. You need to open document to do this"), Amy::ERROR);
	}
	else
	{
		// Save File!
		if (SaveCurrentTab())
		{
			filename= curtab->filename;

			int exitstat;
			std::string sout;
			std::string serr;
			std::string pwd = Glib::getenv("PWD");

			chdir(Glib::path_get_dirname(curtab->filename).c_str());

			Glib::spawn_command_line_sync("sh -c '" + Amy::get_command(Amy::LatexCommand,filename) + "'",&sout,&serr,&exitstat);

			chdir(pwd.c_str());

			AmyEdit::instance()->set_output_text(sout+serr);

			if (exitstat)
			{
				AmyEdit::instance()->pop_up_output();
				filename = "";
			}
		}
	}

	return filename;
}

bool EditTabs::SaveTmpTex()
{
	Page *curtab = get_current_tab();
	if (curtab==NULL)
	{
		Amy::Message(_("Save Failed"), _("Could not create a temporary file. You need to have a document open to perform this action"), Amy::ERROR);
		return false;
	}
	else
	{
		try
		{
			Glib::RefPtr<Glib::IOChannel>outfile = Glib::IOChannel::create_from_file(Glib::build_filename(PrefStruct::instance()->get<Glib::ustring>("General", "config_dir"), "temp.tex"), "w");
			outfile->write(curtab->get_buffer()->get_text());
			outfile->flush();
			return true;
		}
		catch(Glib::FileError error){
			Amy::Message(_("Save Failed"), _("An error occurred whilst attempting to save a temporary file, the system reported this error as ") + error.what(), Amy::ERROR);
			return false;
		}
	}      
}

void EditTabs::SaveAllTabs()
{
	int cur=get_current_page();

	for (int x=0;x<get_n_pages();x++)
	{
		set_current_page(x);
		SaveCurrentTab();
	}

	set_current_page(cur);
}

bool EditTabs::CloseTab(int tab)
{
	extern SigC::Signal1<void,bool> signal_file_open;

	if (tab<0)
		tab=get_current_page();
	Page *curtab = dynamic_cast<Page*>(get_nth_page(tab));
	if (curtab!=NULL)
	{
		if (curtab->get_buffer()->get_modified())
		{

			int close = Amy::Message(curtab->tablabel->get_text()+_(" is unsaved!"),
					_("Do you wish to save it?,\n"
					"If you choose no you will lose\n"
					"all changes since your last save"), 
					Amy::YES_NO, true);

			switch (close)
			{
				case Gtk::RESPONSE_YES:
					SaveCurrentTab();
					break;
				case Gtk::RESPONSE_NO:

					delete(get_nth_page(tab));
					if (get_n_pages()==0)
					{
						SetWindowTitle();
						signal_file_open(false);
					}
					break;
				default:
					return false;
					break;
			}
		}
		else
		{
			delete(get_nth_page(tab));
			if (get_n_pages()==0)
			{
				SetWindowTitle();
				signal_file_open(false);
			}
		}
	}
	else
	{
		Amy::Message(_("Failed to close tab"), _("There is no document open to be closed"), Amy::ERROR);
	}
	return true;
}

bool EditTabs::CloseAllTabs(){
	int cur=get_current_page();
	int max=get_n_pages();

	for (int x=0;x<max;x++)
	{
		set_current_page(0);

		if (!CloseTab())
		{
			set_current_page(cur);
			return false;
		}

	}

	set_current_page(cur);
	return true;
}  

Glib::ustring EditTabs::TabNameFromFile(Glib::ustring start)
{
	start.erase(0,start.rfind('/',start.length())+1);
	return (start);


}

void EditTabs::SaveCurrentTabAsTemplate()
{
	Gtk::Dialog savedialog;
	// Create a dialog with a text entry, to save template as..
	savedialog.set_title("Save Template...");
	savedialog.set_transient_for(*AmyEdit::instance());
	Gtk::VBox *vbox=savedialog.get_vbox();
	Gtk::Entry templatename;

	vbox->add(*Gtk::manage(new Gtk::Label("Enter Name:")));
	vbox->add(templatename);

	savedialog.add_button(Gtk::Stock::OK,Gtk::RESPONSE_OK);
	savedialog.add_button(Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL);

	savedialog.show_all_children();
	if (savedialog.run() == Gtk::RESPONSE_OK)
	{
		try
		{
			Glib::RefPtr<Glib::IOChannel>outfile = Glib::IOChannel::create_from_file(Glib::build_filename(Glib::build_filename(PrefStruct::instance()->get<Glib::ustring>("General", "data_dir"), "templates"), templatename.get_text()), "w");
			outfile->write(get_current_tab()->get_buffer()->get_text());
			outfile->flush();

		}
		catch(Glib::FileError error)
		{
			Amy::Message(_("Template Save Failed"), _("An error occurred whilst attempting to save a template.\nThe system reported this error as: ") + error.what(), Amy::ERROR);
		}
	}


}
Glib::ustring EditTabs::GetUntitledTabName(Glib::ustring start)
{
	int p = get_n_pages();
	int i = 1;
	Glib::ustring name=start;
	bool exists=false;
	do{
		exists=false;
		name=start+Amy::itoa(i);

		for (int x=0;x<p;x++)
		{
			Page *curtab = static_cast<Page*>(get_nth_page(x));

			if (name==curtab->tablabel->get_text())
				exists=true;
		}

		i++;
	}while(exists);

	return name;
}

void EditTabs::Undo()
{
	Glib::RefPtr<Gtk::SourceBuffer> buffer = get_current_view()->get_buffer();
	if (buffer->can_undo ())
		buffer->undo ();

}


void EditTabs::Redo()
{
	Glib::RefPtr<Gtk::SourceBuffer> buffer = get_current_view()->get_buffer();
	if (buffer->can_redo ())
		buffer->redo ();


}

void EditTabs::CutText()
{
	Page *curtab = get_current_tab();
	if (curtab)
	{

		Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
		curtab->get_buffer()->cut_clipboard(refClipboard);

	}

}

void EditTabs::CopyText()
{
	Page *curtab = get_current_tab();
	if (curtab)
	{

		Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
		curtab->get_buffer()->copy_clipboard(refClipboard);

	}
}

void EditTabs::PasteText()
{
	Page *curtab = get_current_tab();
	if (curtab)
	{

		Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
		curtab->get_buffer()->paste_clipboard(refClipboard);

	}

}

void EditTabs::DeleteText()
{
	Page *curtab = get_current_tab();
	if (curtab)
	{

		Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
		curtab->get_buffer()->erase_selection();

	}

}

void EditTabs::FindNext()
{
	Page *curtab = get_current_tab();
	if (curtab)
	{
		Glib::RefPtr<Gtk::SourceBuffer::Mark> cursor=curtab->get_buffer()->get_insert();
		Gtk::SourceBuffer::iterator start, end;
		GtkTextIter climit;
		Gtk::SourceBuffer::iterator limit(&climit);
		FindDialog fdialog;
		int result;
		do
		{

			result=fdialog.run();
			if (result==Gtk::RESPONSE_OK)
			{
				/* 
				   Older versions of Gtk::TextIter::forward_search
				   are buggy, use the C version to prevent lots of 
				   Gtk Warnings 
				   FIXME! Add a check in configure, to define the *better*
				   version if it is available 
				   */

				if(gtk_source_iter_forward_search(cursor->get_iter().gobj(),fdialog.get_text().c_str(),fdialog.get_flags(),start.gobj(),end.gobj(),0))
				{
					curtab->get_buffer()->select_range(end, start);
					(static_cast<Gtk::SourceView*>(curtab->get_child()))->scroll_to(end);
				}
				else
				{
					Amy::Message(fdialog.get_text()+_(" not found!"));
				}

			}
		}while(result==Gtk::RESPONSE_OK );
	}
}

void EditTabs::SpellCheck()
{
	Page *curtab = get_current_tab();
	if (curtab)
	{
		SpellDialog::engage();
						
	}
}



Glib::ustring EditTabs::GetRootFileName()
{
	Page *curtab = get_current_tab();
	if (curtab)
	{
		return curtab->filename;
	}
	return Glib::ustring("");
}



void EditTabs::on_switching_page(GtkNotebookPage *page, guint number)
{
	extern SigC::Signal1<void, int> signal_line_set;
	extern SigC::Signal1<void, bool> signal_file_open;
	Page *curtab = get_current_tab();

	if (curtab)
	{
		signal_line_set(curtab->get_buffer()->get_insert()->get_iter().get_line());
		curtab->get_child()->grab_focus();
		SetWindowTitle();
		signal_file_open(true);

	}

}

Glib::RefPtr <Gtk::SourceBuffer> EditTabs::is_buffer_open(Glib::ustring filename){
	int p = get_n_pages();

	for (int x=0;x<p;x++){
		Page *curtab = static_cast<Page*>(get_nth_page(x));

		if (curtab->filename==filename)
			return curtab->get_buffer();
	}

	return Glib::RefPtr <Gtk::SourceBuffer> (NULL);


}

EditTabs::Page::Page(EditTabs *parent, Glib::RefPtr<Gtk::SourceBuffer> buffer){

	/* Note to self! CLEAN THIS UP!!!! */

	/* Page is actually a ScrollWindow thing */
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);

	Gtk::SourceView *sourceview=Gtk::manage(new Gtk::SourceView());

	if (buffer)
		sourceview->set_buffer(buffer);

	sourceview->modify_font(Pango::FontDescription("monospace"));

	sourceview->set_wrap_mode(Gtk::WRAP_WORD_CHAR);
	add(*sourceview);

	on_preferences_update();

	extern SigC::Signal0<void> signal_preferences_update;
	get_buffer()->signal_modified_changed().connect(sigc::mem_fun(*this, &EditTabs::Page::on_modified_changed));
	get_buffer()->signal_changed().connect(sigc::mem_fun(*this, &EditTabs::Page::on_changed));
	get_buffer()->signal_mark_set().connect(sigc::mem_fun(*this, &EditTabs::Page::on_mark_changed));
	signal_preferences_update.connect(sigc::mem_fun(*this, &EditTabs::Page::on_preferences_update));


	/***********************
	 * Syntax Highlighting *
	 ***********************/

	/* This only needs to happen if the tab isn't already open 
	 * (ie buffer is null) */

	if (!buffer){
		Glib::ustring mime_type("text/x-tex");

		Glib::RefPtr<Gtk::SourceLanguagesManager> lm = buffer->get_languages_manager();
		Glib::RefPtr<Gtk::SourceLanguage> language = lm->get_language_from_mime_type(mime_type);


		if (language) 
			get_buffer()->set_language(language);
	}

	/*********************
	 * The Tab Widget
	 *********************/

	Gtk::HBox *tabwidget=Gtk::manage(new Gtk::HBox());
	tablabel=Gtk::manage(new Gtk::Label());

	Gtk::Button *tabbutton=Gtk::manage(new Gtk::Button());
	tabbutton->add(*Gtk::manage(new Gtk::Image(Gtk::StockID("gtk-close"), Gtk::IconSize(Gtk::ICON_SIZE_MENU))));

	int w,h;
	Gtk::IconSize::lookup(Gtk::ICON_SIZE_MENU,w,h);

	tabbutton->set_size_request(w+2,h+2);
	tabbutton->set_relief(Gtk::RELIEF_NONE);
	tabbutton->signal_clicked().connect(sigc::mem_fun(*this,&EditTabs::Page::on_close_me));

	tabwidget->add(*tablabel);
	tabwidget->pack_start(*tabbutton,Gtk::PACK_SHRINK);
	tabwidget->show_all_children();

	/* end tab widget */

	parent->append_page(*this,*tabwidget);
	parent->show_all_children();

}

void EditTabs::Page::on_close_me(){
	EditTabs *parent = get_parent();
	parent->CloseTab(parent->page_num(*this));
}

void EditTabs::Page::on_modified_changed(){
	EditTabs *parent = get_parent();
	if (filename=="")
	{
		if (get_buffer()->get_modified())
			tablabel->set_text(parent->GetUntitledTabName(_("[Modified] ")));
		else
			tablabel->set_text(parent->GetUntitledTabName(_("[Untitled] ")));
	}
	else
	{
		if (get_buffer()->get_modified())
			tablabel->set_text(parent->TabNameFromFile(filename)+"*");
		else
			tablabel->set_text(parent->TabNameFromFile(filename));
	}
	parent->SetWindowTitle();
}


void EditTabs::Page::on_changed()
{
	extern SigC::Signal1<void, int> signal_line_set;
	signal_line_set(get_buffer()->get_insert()->get_iter().get_line());

}

void EditTabs::Page::on_mark_changed(Gtk::SourceBuffer::iterator iter, Glib::RefPtr<Gtk::SourceBuffer::Mark> mark){
	if (mark->get_name()=="insert")
	{
		extern SigC::Signal1<void, int> signal_line_set;

		signal_line_set(get_buffer()->get_insert()->get_iter().get_line());

	}
}

EditTabs *EditTabs::Page::get_parent()
{
	return (static_cast<EditTabs*>(Gtk::Widget::get_parent()));
}

Gtk::SourceView* EditTabs::Page::get_child()
{
	return (static_cast<Gtk::SourceView*>(Gtk::Bin::get_child()));
}

Glib::RefPtr<Gtk::SourceBuffer> EditTabs::Page::get_buffer()
{
	return (get_child()->get_buffer());
}

EditTabs::Page *EditTabs::get_current_tab()
{
	return (static_cast<EditTabs::Page*>(get_nth_page(get_current_page())));
}

Gtk::SourceView *EditTabs::get_current_view()
{
	return (static_cast<Gtk::SourceView*>(get_current_tab()->get_child()));
}


void EditTabs::Page::on_preferences_update()
{
	// For things that need to happen for all pages
	PrefStruct *prefs = PrefStruct::instance();
	
	get_child()->set_tabs_width(prefs->get<int>("General", "tab_size"));
	get_buffer()->set_highlight(prefs->get<bool>("General", "syntax_highlighting"));

}


void EditTabs::on_preferences_update()
{
	// For things that need to happen for the EditTabs object

}

void EditTabs::SetWindowTitle()
{
	EditTabs::Page* curtab=get_current_tab();
	if (!curtab)
	{
		get_parent_window()->set_title(Glib::ustring("AmyEdit ")+VERSION);
	}
	else
	{
		get_parent_window()->set_title(Glib::ustring("AmyEdit ")+VERSION+" - " + curtab->tablabel->get_text());
	}
}

