/*****************************************************************************
    AmyEdit - A GTKMM LaTeX Editor
    Copyright (C) 2004-2005 Rob Page
    email:  page <dot> rob <at> gmail <dot> com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "AmyEdit.hh"
#include "NewDialog.hh"
#include "PrefStruct.hh"
#include "Utils.hh"

#include <gtkmm/button.h>
#include <gtkmm/label.h>
#include <gtkmm/stock.h>
#include <gtkmm/separator.h>
#include <gtkmm/alignment.h>
#include <gtkmm/entry.h>
#include <fstream>

NewDialog::NewDialog():
	a4paper(_("a4paper")),
	a5paper(_("a5paper")),
	article(_("article")),
	report(_("report")),
	parindent(_("Indent Paragraphs")),
	doublespacing(_("Double Spacing")),
	listbox(2)
{  
	set_transient_for(*AmyEdit::instance());

	/* Standard Tab */
	populate_listbox();

	Gtk::VBox *box = Gtk::manage(new Gtk::VBox());
	listbox.show_column(_("Templates"),0);
	box->add(listbox);

	tabs.append_page(*box, _("Standard"));

	/* Advanced Tab */

	Gtk::RadioButton::Group paper_group;
	Gtk::RadioButton::Group doctype_group;

	a4paper.set_group(paper_group);
	a5paper.set_group(paper_group);

	article.set_group(doctype_group);
	report.set_group(doctype_group);

	parindent.set_active();

	Gtk::VBox *advbox = Gtk::manage(new Gtk::VBox());
	Gtk::Button *newbutton = Gtk::manage(new Gtk::Button(_("Save Template")));

	advbox->add(*Gtk::manage(new Gtk::Label(_("Paper Type"))));
	advbox->add(a4paper);
	advbox->add(a5paper);
	advbox->add(*Gtk::manage(new Gtk::HSeparator()));
	advbox->add(*Gtk::manage(new Gtk::Label(_("Document Type"))));
	advbox->add(article);
	advbox->add(report);
	advbox->add(*Gtk::manage(new Gtk::HSeparator()));
	advbox->add(*Gtk::manage(new Gtk::Label(_("Misc Settings"))));
	advbox->add(parindent);
	advbox->add(doublespacing);
	advbox->add(*Gtk::manage(new Gtk::HSeparator()));
	advbox->add(*newbutton);

	newbutton->signal_clicked().connect(sigc::mem_fun(*this, &NewDialog::on_save_template));

	tabs.append_page(*advbox, _("Advanced"));

	get_vbox()->add(tabs);
	add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

	set_title(_("New Document"));
	set_skip_taskbar_hint();
	show_all_children();

	/* TODO Work out why this doesn't work if it happens sooner... 
	 * ie: with the other listbox initialisation */
	listbox.select_first();
}

NewDialog::~NewDialog()
{ 

}

Glib::ustring NewDialog::getDoc()
{
	Glib::ustring tabtext=tabs.get_tab_label_text(*tabs.get_nth_page(tabs.get_current_page()));
	if (tabtext==_("Standard"))
	{
		Glib::ustring filename=listbox.get_selected(1);
		if (filename!="")
			return Glib::file_get_contents(filename);
		else 
			return Glib::ustring("");
	}
	else 
	{
		return getDocString();
	}
}

Glib::ustring NewDialog::getDocString()
{
	Glib::ustring docString="";

	docString+= "\\documentclass[";

	if (a4paper.get_active())
		docString+="a4paper";
	else if (a5paper.get_active())
		docString+="a5paper";

	docString+= "]{";

	if (article.get_active())
		docString +="article";
	else if (report.get_active())
		docString +="report";

	docString+= "}\n";

	if (!parindent.get_active() || doublespacing.get_active())
		docString +="\\usepackage{setspace}\n";

	if(!parindent.get_active())
		docString +="\\setlength{\\parindent}{0pt}\n\\setlength{\\parskip}{\\baselineskip}\n";

	if(doublespacing.get_active())
		docString +="\\doublespacing{}\n";

	docString +="\\begin{document}\n";

	docString +="\n\n\n";

	docString +="\\end{document}\n";

	return docString;

}

void NewDialog::populate_listbox()
{
	std::string local_templates_dir = Glib::build_filename(PrefStruct::instance()->get<Glib::ustring>("General", "data_dir"), "templates");
	Amy::populate_text_list_model_from_dir(listbox.get_model(), local_templates_dir, true, true);
	
	std::list<std::string> list = Glib::ArrayHandle<std::string>(g_get_system_data_dirs());
	std::list<std::string>::iterator iter;
	std::string template_dir_suffix = Glib::build_filename("amyedit", "templates");
	
	for (iter = list.begin(); iter!=list.end();iter++)
	{
		std::string templates_dir = Glib::build_filename(*iter, template_dir_suffix);
		Amy::populate_text_list_model_from_dir(listbox.get_model(), templates_dir, false, true);
	}
}

void NewDialog::on_save_template()
{
	Gtk::Dialog savedialog;
	savedialog.set_transient_for(*this);
	// Create a dialog with a text entry, to save template as..
	savedialog.set_title(_("Save Template"));
	Gtk::VBox *vbox=savedialog.get_vbox();
	Gtk::Entry templatename;

	vbox->add(*Gtk::manage(new Gtk::Label(_("Enter Template Name:"))));
	vbox->add(templatename);

	savedialog.add_button(Gtk::Stock::CANCEL,Gtk::RESPONSE_CANCEL);
	savedialog.add_button(Gtk::Stock::OK,Gtk::RESPONSE_OK);

	savedialog.show_all_children();
	if (savedialog.run() == Gtk::RESPONSE_OK)
	{
		try
		{
			Glib::RefPtr<Glib::IOChannel>outfile = Glib::IOChannel::create_from_file(Glib::build_filename(Glib::build_filename(PrefStruct::instance()->get<Glib::ustring>("General", "data_dir"), "templates"), templatename.get_text()), "w");
			outfile->write(getDocString());
			outfile->flush();
		}
		catch(Glib::FileError error)
		{
			Amy::Message(_("Template Save Failed"), _("An error occurred whilst attempting to save a template.\nThe system reported this error as: ") + error.what(), Amy::ERROR);
		}
	}
	populate_listbox();
}

