/*****************************************************************************
	AmyEdit - A GTKMM LaTeX Editor
	Copyright (C) 2004-2005 Rob Page
	email:  page <dot> rob <at> gmail <dot> com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 *****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "SpellDialog.hh"
#include "Utils.hh"
#include "PrefStruct.hh"

#include <gtkmm/table.h>
#include <gtkmm/stock.h>
#include <gtkmm/label.h>
#include <gtkmm/entry.h>
#include <gtkmm/liststore.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/button.h>

#include <aspell.h>
#include <iostream>

void SpellDialog::init(){
	SpellDialog::get_config();

}

void SpellDialog::deinit(){
	AspellConfig* config = get_config();
	if (config)
		delete_aspell_config(config);
}

AspellConfig* SpellDialog::get_config(){
	static AspellConfig* config = NULL;

	if (config == NULL){
		config = new_aspell_config();
		aspell_config_replace(config, "lang", getPreferences()->get<Glib::ustring>("General", "language").c_str());
		aspell_config_replace(config, "mode", "tex");

		if (aspell_config_error(config) != 0) {
			Amy::Message("Error setting up spellcheck config", Glib::ustring("The spellchecker could not be initialised. The system reported this error as: ") + aspell_config_error_message(config), Amy::ERROR);
		}
	}
	return config;

}

SpellDialog::SpellDialog()
{
	set_title(_("Spell Check"));

	misslabel.set_sensitive(false);

	Gtk::Table *table=Gtk::manage(new Gtk::Table(6,4));
	table->set_spacings(5);
	table->set_homogeneous();

	Gtk::Button *addbutton = Gtk::manage(new Gtk::Button(_("Add")));
	Gtk::Button *checkbutton = Gtk::manage(new Gtk::Button(_("Check")));
	Gtk::Button *skipbutton = Gtk::manage(new Gtk::Button(_("Skip")));
	Gtk::Button *skipallbutton = Gtk::manage(new Gtk::Button(_("Skip All")));
	Gtk::Button *replacebutton = Gtk::manage(new Gtk::Button(_("Replace")));
	Gtk::Button *replaceallbutton = Gtk::manage(new Gtk::Button(_("Replace All")));

	table->attach(*Gtk::manage(new Gtk::Label(_("Word:"))), 0,1,0,1);
	table->attach(misslabel,1,3,0,1);
	table->attach(*addbutton,3,4,0,1);

	table->attach(*Gtk::manage(new Gtk::Label(_("Replace With:"))), 0,1,1,2);
	table->attach(replaceentry,1,3,1,2);
	table->attach(*checkbutton,3,4,1,2);

	listbox.show_column(_("Alternatives"));
	
	Gtk::ScrolledWindow *altwindow = Gtk::manage(new Gtk::ScrolledWindow());
	altwindow->set_policy(Gtk::POLICY_NEVER,Gtk::POLICY_ALWAYS);
	altwindow->add(listbox);
	altwindow->set_shadow_type(Gtk::SHADOW_ETCHED_IN);
	table->attach(*altwindow,0,2,2,6);

	table->attach(*skipbutton,2,3,3,4);
	table->attach(*skipallbutton,3,4,3,4);
	table->attach(*replacebutton,2,3,4,5);
	table->attach(*replaceallbutton,3,4,4,5);

	get_vbox()->pack_start(*table,Gtk::PACK_EXPAND_WIDGET,5);
	add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CANCEL);

	listbox.get_selection()->signal_changed().
		connect(sigc::mem_fun(*this, &SpellDialog::on_selection_changed));

	addbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_add));
	checkbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::get_suggestions));
	skipbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_skip));
	skipallbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_skipall));
	replacebutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_replace));
	replaceallbutton->signal_clicked().connect(sigc::mem_fun(*this, &SpellDialog::on_replaceall));

	set_skip_taskbar_hint();
	show_all_children();
}

SpellDialog::~SpellDialog()
{
	if (speller){
		aspell_speller_save_all_word_lists(speller);
		delete_aspell_speller(speller);
	}
	if (checker){
		delete_aspell_document_checker(checker);
	}
}

void SpellDialog::setCheckArea(Glib::RefPtr<Gtk::TextMark> areastart, Glib::RefPtr<Gtk::TextMark> areaend)
{

	current_string=areastart->get_iter().get_text(areaend->get_iter());
	aspell_document_checker_process(checker, current_string.c_str(), -1);

}

AspellToken SpellDialog::getNextMiss()
{
	AspellToken miss=aspell_document_checker_next_misspelling(checker);

	if (miss.len == 0)
	{
		return miss;
	}
	else
	{
		Glib::ustring missword (current_string,miss.offset,miss.len);
		misslabel.set_text(missword);
		replaceentry.set_text(missword);

		return miss;

	}
}

void SpellDialog::on_preferences_update()
{
	aspell_config_replace(get_config(), "lang", getPreferences()->get<Glib::ustring>("General", "language").c_str());
}

void SpellDialog::on_selection_changed()
{
	replaceentry.set_text(listbox.get_selected());
}

void SpellDialog::on_replace(){
	aspell_speller_store_replacement(speller, misslabel.get_text().c_str(), -1,
			replaceentry.get_text().c_str(), -1);
	ret=replaceentry.get_text();
	response(Gtk::RESPONSE_YES);
}

void SpellDialog::on_replaceall(){
	aspell_speller_store_replacement(speller, misslabel.get_text().c_str(), -1,
			replaceentry.get_text().c_str(), -1);
	skiprep[misslabel.get_text()]=replaceentry.get_text();
	ret=replaceentry.get_text();
	response(Gtk::RESPONSE_YES);
}

void SpellDialog::on_skip(){
	response(Gtk::RESPONSE_NO);
}

void SpellDialog::on_skipall(){
	skiprep[misslabel.get_text()]=Glib::ustring("");
	response(Gtk::RESPONSE_NO);
}

void SpellDialog::on_add(){
	aspell_speller_add_to_personal(speller, misslabel.get_text().c_str(), -1);
	response(Gtk::RESPONSE_NO);
}

void SpellDialog::get_suggestions(){
	Glib::ustring word=replaceentry.get_text();
	listbox.get_model()->clear();
	const AspellWordList * suggestions = aspell_speller_suggest(speller,word.c_str(), -1);
	AspellStringEnumeration * elements = aspell_word_list_elements(suggestions);
	const char *words;
	while ((words = aspell_string_enumeration_next(elements)) != NULL )
	{
		listbox.get_model()->append(words);
	}
	delete_aspell_string_enumeration(elements);
}

int SpellDialog::engage(){

	SpellDialog dialog;
	return dialog.checkDocument();
}

int SpellDialog::checkDocument(){

	speller = 0;
	checker = 0;
	AspellCanHaveError * possible_err = new_aspell_speller(get_config());
	if (aspell_error_number(possible_err) != 0)
		puts(aspell_error_message(possible_err));
	else
		speller = to_aspell_speller(possible_err);
	if (speller==NULL){
		Amy::Message(_("Could not Initialise Spell Checker!"), _("AmyEdit was unable to initialise the spell checker configuration. This probably means you need to install a dictionary, many are available at ftp://ftp.gnu.org/gnu/aspell/dict/\nAlternatively try choosing a different language in Preferences"), Amy::WARNING);
		return -1;
	}

	possible_err = new_aspell_document_checker(speller);
	if (aspell_error_number(possible_err) != 0)
		puts(aspell_error_message(possible_err));
	else
		checker = to_aspell_document_checker(possible_err);

	if (checker==NULL){
		Amy::Message(_("Could not Initialise Spell Checker!"), _("AmyEdit was unable to initialise the spell checker configuration."), Amy::WARNING);
		return -1;
	}
	aspell_document_checker_reset(checker);

	view = getCurrentPage()->get_child();
	buffer = view->get_buffer();

	buffer->set_check_brackets(false);

	/* areastart has left gravity, arearight has right gravity, as inserts only
	 * occur inside them, they should always mark the beginning and ends of the
	 * area currently being checked, ie: a line */
	
	Glib::RefPtr<Gtk::TextMark> areastart = buffer->create_mark(buffer->begin(), true);
	Glib::RefPtr<Gtk::TextMark> areaend = buffer->create_mark(buffer->begin(), false);


	/* Blurgh Variable */

	bool not_end_of_doc;

	/* SpellCheck the Document */
	do
	{
		delta_area = 0;

		Gtk::TextIter line_end = areastart->get_iter();
		line_end.forward_to_line_end();
		buffer->move_mark(areaend, line_end);
		
		setCheckArea(areastart, areaend);
		AspellToken miss = getNextMiss();

		while(miss.len != 0)
		{
			if (changeWord(miss, areastart->get_iter())==false)
				return 0;
			miss = getNextMiss();
		}
	
		Gtk::TextIter line_start = areaend->get_iter();
		not_end_of_doc = line_start.forward_line();
		buffer->move_mark(areastart, line_start);
		
	} while(not_end_of_doc);
	

	Amy::Message(_("SpellCheck Complete!"));
	buffer->place_cursor(areastart->get_iter());

	buffer->delete_mark(areastart);
	buffer->delete_mark(areaend);
	
	buffer->set_check_brackets(true);
	return 0;	
}


// Return true for everything except user clicking close
bool SpellDialog::changeWord(AspellToken miss, Gtk::TextIter wordstart){

	wordstart.forward_chars(miss.offset+delta_area);
	Gtk::TextIter wordend = wordstart; 
	wordend.forward_chars(miss.len);

	buffer->select_range(wordstart,wordend);
	view->scroll_to(buffer->get_insert(),0.2);

	int response;
	
	if (skiprep.find(misslabel.get_text())!=skiprep.end())
	{
		ret=skiprep[misslabel.get_text()];
		if (ret=="")
			response = Gtk::RESPONSE_NO;
		else
			response = Gtk::RESPONSE_YES;

	} 
	else
	{

		misslabel.set_text(misslabel.get_text());
		get_suggestions();
		listbox.select_first();
		response = Gtk::Dialog::run();
	}


	if (response==Gtk::RESPONSE_NO)
	{
		/* Skip Clicked */
	}
	else if(response==Gtk::RESPONSE_YES)
	{
		/* Replace Clicked */
		
		buffer->select_range(wordstart,wordend);
		delta_area = delta_area - buffer->get_slice(wordstart,wordend).size();
		buffer->erase_selection();
		buffer->insert_at_cursor(ret);
		delta_area += ret.size();
	}
	else
	{
		/* Close Clicked */
		
		buffer->place_cursor(wordend);
		buffer->set_check_brackets();
		return false;
	}

	return true;
}

