/*****************************************************************************
    AmyEdit - A GTKMM LaTeX Editor
    Copyright (C) 2004-2005 Rob Page 
    email:  page <dot> rob <at> gmail <dot> com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*****************************************************************************/

#ifdef USE_PRECOMPILED_HEADERS
#  include "all.hh"
#endif

#include "config.h"

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

#include "Utils.hh"
#include "Utils_p.hh"
#include "AmyEdit.hh"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>
#include <sstream>

namespace Amy{
	
	Gtk::MessageType MiniDialog::getMessageType(DialogType type){
		switch(type)
		{
			case INFO:
				return Gtk::MESSAGE_INFO;
			case WARNING:
				return Gtk::MESSAGE_WARNING;
			case ERROR:
				return Gtk::MESSAGE_ERROR;
			case OK_CANCEL:
				return Gtk::MESSAGE_QUESTION;
			case YES_NO:
				return Gtk::MESSAGE_QUESTION;
			case MISC:
				return Gtk::MESSAGE_QUESTION;
		}

		// TODO This should throw an exception
		return Gtk::MESSAGE_INFO;
	}
	
	Gtk::ButtonsType MiniDialog::getButtonType(DialogType type){
		switch(type)
		{
			case INFO:
				return Gtk::BUTTONS_OK;
			case WARNING:
				return Gtk::BUTTONS_CLOSE;
			case ERROR:
				return Gtk::BUTTONS_CLOSE;
			case OK_CANCEL:
				return Gtk::BUTTONS_OK_CANCEL;
			case YES_NO:
				return Gtk::BUTTONS_YES_NO;
			case MISC:
				return Gtk::BUTTONS_NONE;
		}
		// TODO This should throw an exception
		return Gtk::BUTTONS_OK;
	}
	
	MiniDialog::MiniDialog(Glib::ustring heading, Glib::ustring body, DialogType type):
		Gtk::MessageDialog(heading, false, getMessageType(type), getButtonType(type))
	{
		AmyEdit* amyedit = AmyEdit::instance();

		if (amyedit)
			set_transient_for(*amyedit);

		set_secondary_text(body);
		set_skip_taskbar_hint();
		set_skip_pager_hint();

	}

	int Message(Glib::ustring heading, Glib::ustring body, DialogType type, bool add_cancel){
		MiniDialog dialog(heading,body,type);
		if (add_cancel)
			dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
		return dialog.run();
	}

	Glib::ustring itoa(int num){
		std::ostringstream ostr;
		ostr << num;
		Glib::ustring retVal = ostr.str();
		return retVal;
	}

	void populate_text_list_model_from_dir(Glib::RefPtr<TextListModel> model, const std::string& dirname, bool clear_list, bool store_full_path){
		if (clear_list)
			model->clear();

		if (!Glib::file_test(dirname, Glib::FILE_TEST_IS_DIR))
			return;
		
		Glib::Dir dir(dirname);
		Glib::ustring command;

		while ((command=Glib::filename_to_utf8(dir.read_name())) != "") {
			Glib::ustring full_filename = Glib::build_filename(dirname, command);
			if (Glib::file_test(filename_to_utf8(full_filename), Glib::FILE_TEST_IS_REGULAR))
			{
				Gtk::TreeModel::Row row = model->append();
				model->set_value(command, row, 0);
				if (store_full_path)
					model->set_value(full_filename, row, 1);
			}
		}
	
	}

	void populate_text_list_model_from_list(Glib::RefPtr<TextListModel> model, const Glib::ArrayHandle<Glib::ustring>& list, bool clear_list)
	{
		if (clear_list)
			model->clear();

		Glib::ArrayHandle<Glib::ustring>::iterator iter = list.begin();
		for (iter = list.begin(); iter != list.end(); iter++)
		{
			model->append(*iter);
		}
	}

	Glib::ustring get_command(CommandType type, const Glib::ustring& filename)
	{

		PrefStruct* prefs = PrefStruct::instance();
		Glib::ustring key = prefs->get<Glib::ustring>("General", "preview_combo");
		std::vector<Glib::ustring> cmds = prefs->get<Glib::ArrayHandle<Glib::ustring> >("Commands", key);

		Glib::ustring command;
		
		if (type==LatexCommand)
			command = cmds.at(0);
		else if (type==PreviewCommand)
			command = cmds.at(1);
		else 
			/* TODO This should throw a custom AmyEdit exception */
			throw Glib::MarkupError(Glib::MarkupError::UNKNOWN_ELEMENT,"An impossible value was passed to Amy::get_command");

		Glib::ustring output_file = cmds.at(2);
		
		substitute(command, filename, &output_file);
		return command;
	}
	
	void substitute(Glib::ustring& string, const Glib::ustring& filename, Glib::ustring* output_file)
	{
		Glib::ustring filename_sans_tex = filename;
	
		if (output_file) substitute(*output_file, filename);
		
		if (filename.substr(filename.size()-4)==".tex") {
			filename_sans_tex.erase(filename.size()-4);
		}

		Glib::ustring::size_type repint = 0;
		while ((repint=string.find("%", repint))!=std::string::npos)
		{
			if (string.at(repint+1)=='f') {
				/* f means insert the full filename */
				string.erase(repint,2);
				string.insert(repint,filename);
				repint += filename.length()-2;
			} else if (string.at(repint+1)=='g') {
				/* g means insert the filename less the .tex extension */
				string.erase(repint,2);
				string.insert(repint, filename_sans_tex);
				repint += filename_sans_tex.length()-1;
			} else if (string.at(repint+1)=='h' && output_file) {
				/* h means insert the output filename */
				string.erase(repint,2);
				string.insert(repint, *output_file);
				repint += output_file->length() -1;
			} else if (string.at(repint+1)=='%') {
				/* % means replace with a % */
				string.erase(repint,1);
				repint++;
			}
		}

	}
}
